package cn.timer.api.controller.insure;

import cn.timer.api.bean.insure.InsureApplicant;
import cn.timer.api.bean.insure.InsureLog;
import cn.timer.api.bean.insure.InsurePolicy;
import cn.timer.api.bean.insure.InsureUser;
import cn.timer.api.bean.qyzx.QyzxEntInfoM;
import cn.timer.api.bean.yggl.YgglMainEmp;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.controller.insure.bean.ExcelBean;
import cn.timer.api.controller.insure.enums.InsuranceEnum;
import cn.timer.api.dao.insure.InsurePolicyMapper;
import cn.timer.api.dao.insure.InsureUserMapper;
import cn.timer.api.dao.yggl.YgglMainEmpMapper;
import cn.timer.api.dto.insure.InsureDto;
import cn.timer.api.dto.insure.PolicyDto;
import cn.timer.api.utils.*;
import com.alibaba.druid.util.Base64;
import com.aliyun.oss.common.utils.StringUtils;
import com.beust.jcommander.internal.Lists;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.google.common.collect.Maps;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.models.auth.In;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.ibatis.annotations.Param;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.eclipse.jetty.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * @Description TODO
 * @Author wgd
 * @Date 2022/3/1 11:34
 */
@Api(tags = "8.0保险列表")
@RestController
@Transactional
@RequestMapping(value = "/insure", produces = {"application/json"})
public class InsureContorll {
    private static final Logger log = LoggerFactory.getLogger(InsureContorll.class);
    /*测试用*/
    private static String appid = "1002303100602312445";
    /*测试用*/
    private static String secret = "acb329868c31d5b3ba03de40dac13dd9";

    /*保全测试用*/
    private static String appidq = "1000115041006006938";
    /*保全测试用*/
    private static String secretq = "6ba1bf4aa0bd14368c95ff0b9934a17b";


    /*测试用投保上传文件地址*/
    private static String uploadUrl = " http://sandbox.portal.unistar-ins.com/service/Home/Index/fileUpload";
    /*测试用投保申请地址*/
    private static String insuredUrl = "http://sandbox.portal.unistar-ins.com/mall/Home/Index/createQuotePolicy";
    /*测试用保全上传文件地址*/
    private static String uploadUrl2 = "http://sandbox.portal.unistar-ins.com/fuli/Home/Index/file_upload";
    /*测试用保全申请地址*/
    private static String batchUrl = "http://sandbox.portal.unistar-ins.com/fuli/Home/WelfareProduct/batch_declare";

    private static SimpleDateFormat  dtf3 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Autowired
    private YgglMainEmpMapper ygglMainEmpMapper;

    @Autowired
    private InsureUserMapper insureUserMapper;

    @Autowired
    private InsurePolicyMapper insurePolicyMapper;


    /**
     * 设置请求参数
     *
     * @param sign
     * @return
     */
    private Map setParams(String sign, String appid, String secret) {
        /*当前时间戳*/
        long timestamp = System.currentTimeMillis() / 1000;
        log.info("时间戳" + timestamp);
        Map paramsMap = Maps.newHashMap();
        paramsMap.put("pid", appid);
        paramsMap.put("timestamp", String.valueOf(timestamp));
        paramsMap.put("trace_id", appid + timestamp + new Random().nextInt((9999 - 100) + 1) + 10);
//        paramsMap.put("sign",Md5.md5(appid+secret+timestamp+sign.trim()));

        String value = appid + secret + timestamp + sign;
        log.info("body参数======" + sign);
        paramsMap.put("sign", DigestUtils.md5Hex(value));
        log.info("params参数======" + JSONObject.toJSONString(paramsMap));
        return paramsMap;
    }

    @PostMapping(value = "/insured_add")
    @ApiOperation(value = "1.投保申请", httpMethod = "Post", notes = "投保申请")
    @ApiOperationSupport(order = 2)
    public Result<Object> insured_add(@RequestBody InsureDto insureDto) throws ParseException {
        QyzxEntInfoM qyzxEntInfoM = QyzxEntInfoM.builder().id(Integer.parseInt(insureDto.getOrgCode())).build().selectById(); // 企业信息
        DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
        LocalDateTime now = LocalDateTime.now();
        Date date = Date.from(now.atZone(ZoneId.systemDefault()).toInstant());
        Map bodyMap = Maps.newHashMap();
        ArrayList<Map> quotationsArry = new ArrayList<Map>();
        bodyMap.put("is_ss", 0);
        Map quotations = Maps.newHashMap();
        Integer price = getPrice(insureDto.getBenefitBasicPlan(),insureDto.getBenefitOccupationCategory());
        if(price==0){
            return ResultUtil.error("报价不存在");
        }
        quotations.put("trans_id", "MB_" + dtf2.format(now));/*报价请求流水号*/
        quotations.put("applicant_type", "2");/*投保人类型：2-企业*/
        quotations.put("quotation_type", 3);/*报价类型：3-投保申请*/
        quotations.put("currency", "5");/*币种：5人民币*/
        quotations.put("source_type", "");/*币种：5人民币*/
        quotations.put("cps_email", "");/*币种：5人民币*/
        quotations.put("cps_tel", "");/*币种：5人民币*/
        quotations.put("last_serial_number", "");/*币种：5人民币*/
        quotations.put("from_source", "");/*币种：5人民币*/
        quotations.put("operator_name", "");/*币种：5人民币*/


        /*企业信息*/
        InsureApplicant insureApplicant = InsureApplicant.builder().id(1).build().selectById();
        if (insureApplicant == null) {
            return ResultUtil.error("未设置投保人");
        }
        Map companyApplicantMap = Maps.newHashMap();
        companyApplicantMap.put("applicant_e_name", insureApplicant.getApplicantEName());/*企业/网点名称*/
        companyApplicantMap.put("applicant_e_address", insureApplicant.getApplicantEAddress());/*企业/网点地址*/
        companyApplicantMap.put("applicant_e_contacts", insureApplicant.getApplicantEContacts());/*企业授权操作人姓名*/
        companyApplicantMap.put("applicant_e_phone", insureApplicant.getApplicantEPhone());/*操作人的联系电话*/
        companyApplicantMap.put("applicant_e_no_type", insureApplicant.getApplicantENoType());/*企业证件类型*/
        companyApplicantMap.put("applicant_e_no", insureApplicant.getApplicantENo());/*企业证件号码*/
        companyApplicantMap.put("applicant_e_mail", insureApplicant.getApplicantEMail());/*企业邮箱*/
        companyApplicantMap.put("applicant_province_city", "21721");/*省市区*/
        /*人员清单填被保人员*/
        String url = creaXSSFWorkbook(insureDto.getUserIds(),price);
        if (!StringUtil.isNotBlank(url)) {
            return ResultUtil.error("文件错误");
        }
        companyApplicantMap.put("applicant_employee_list", url);
        quotations.put("company_applicant", companyApplicantMap);


        Map basicMap = Maps.newHashMap();
        basicMap.put("partner_pid", "1231");/*分销商pid*/
        basicMap.put("plan_code_id", "61b85207b73d2ab09fb2959");/*计划代码,选项值:61b85207b73d2ab09fb2959 (生产环境) 61b85207b73d2ab09fb2959 (沙箱环境)*/
        basicMap.put("product_code_id", "61b85207a9f87da19fb5986");/*产品代码,选项值:61b85207a9f87da19fb5986 (生产环境) 61b85207a9f87da19fb5986 (沙箱环境)*/
        basicMap.put("policy_date_start",insureDto.getPolicyDateStart());
        basicMap.put("policy_date_end", insureDto.getPolicyDateEnd());
        basicMap.put("channel", "");
        basicMap.put("fee", "");
        basicMap.put("partner_user_rate", "");
        basicMap.put("promotion_fee", "");
        basicMap.put("agent_id", "");
        basicMap.put("policy_insure_notify", "");
        basicMap.put("promotion_id", 0);
        basicMap.put("policy_number_last_year", "");
        quotations.put("basic", basicMap);
        quotations.put("individual_applicant", Maps.newHashMap());




        /*被保人员 填被保企业信息*/
        ArrayList<Map> insuredArray = new ArrayList<>();
        Map insured = Maps.newHashMap();
        insured.put("trans_id", "PI_" + qyzxEntInfoM.getId() + "_" + dtf2.format(now));
        insured.put("price", insureDto.getUserIds().length * price);/*根据基础计划生成*/
        insured.put("insured_applicant_relations", "36960");
        insured.put("insured_mobile", qyzxEntInfoM.getPhone());
        insured.put("insured_e_contact", qyzxEntInfoM.getOperName());
        insured.put("insured_name", qyzxEntInfoM.getName());
        insured.put("insured_no_type", "3306");
        insured.put("insured_no", qyzxEntInfoM.getCreditCode());
        insured.put("insured_zm_openid", "");
        insured.put("benefit_basic_plan", insureDto.getBenefitBasicPlan());/*基础计划写死*/
        insured.put("benefit_occupation_category", insureDto.getBenefitOccupationCategory());
        insured.put("benefit_EL_employee_number_t", insureDto.getUserIds().length);
        insuredArray.add(insured);
        quotations.put("insured", insuredArray);
        quotations.put("file", Lists.newArrayList());
        quotations.put("specifically_agreed_file", Lists.newArrayList());
        quotations.put("policy_no", "");
        quotations.put("callback_policy", "/callBack/policy/insuredCallBack");
        quotations.put("insured", insuredArray);
        quotationsArry.add(quotations);
        bodyMap.put("quotations", quotationsArry);
        log.info(JSONObject.toJSONString(bodyMap));
        Map dataMap = requestAgent(bodyMap);

        if (dataMap != null) {
            Object o = dataMap.get("data");
            Map<String, Object> map = JSONObject.parseObject(JSONObject.toJSONString(o));
            List<YgglMainEmp> ygglMainEmpList = ygglMainEmpMapper.selectListByIds(insureDto.getUserIds());
            Map policyMap = JSONObject.parseObject(JSONObject.toJSONString(map.get("policy")), Map.class);
            InsureUser insureUser;
            InsurePolicy insurePolicy = InsurePolicy.builder().build();
            insurePolicy.setPolicyDateStart(dtf3.parse(insureDto.getPolicyDateStart()));
            insurePolicy.setPolicyDateEnd(dtf3.parse(insureDto.getPolicyDateEnd()));
            insurePolicy.setPolicyNo(policyMap.get("policy_no").toString());
            insurePolicy.setType(2);/*年单或者月单*/
            insurePolicy.setCurrency("5");
            insurePolicy.setStatus("1");
            insurePolicy.setPolicyFile(policyMap.get("policy_file").toString());
            insurePolicy.setKitUrl(policyMap.get("kit_url").toString());
            insurePolicy.setOrgCode(qyzxEntInfoM.getId());
            insurePolicy.setCreateTime(date);
            insurePolicy.setUpdateTime(date);
            insurePolicy.setSchemeName("【新】-安诚-雇主责任险（劳务公司）预付款");/*先写死*/
            insurePolicy.setTotalPremium(String.valueOf(insureDto.getUserIds().length * price));
            insurePolicy.insert();
            for (YgglMainEmp y : ygglMainEmpList) {
                insureUser = InsureUser.builder().build();
                insureUser.setUserId(y.getId());
                insureUser.setInsuredName(qyzxEntInfoM.getName());
                insureUser.setPrice(price.toString());/*前端获取的保费*/
                insureUser.setTransId("MB_" + dtf2.format(now));
                insureUser.setPremium(String.valueOf(ygglMainEmpList.size() * price));
                insureUser.setBatchNo("P_" + qyzxEntInfoM.getId() + "_" + dtf2.format(now));
                insureUser.setBenefitBasicPlan(insureDto.getBenefitBasicPlan());
                insureUser.setPolicyNo(policyMap.get("policy_no").toString());
                insureUser.setUserId(y.getId());
                insureUser.setOrgCode(y.getOrgCode());
                insureUser.setBenefitOccupationCategory(insureDto.getBenefitOccupationCategory());
                insureUser.setInsuredMobile(y.getPhone());
                insureUser.setInsuredNo(y.getZjNum());
                insureUser.setInsuredEContact(y.getName());
                insureUser.setApplicantEmployeeList(url);
                insureUser.setInsureStatus(1);
                insureUser.setPolicyDateStart(dtf3.parse(insureDto.getPolicyDateStart()));
                insureUser.setPolicyDateEnd(dtf3.parse(insureDto.getPolicyDateEnd()));
                insureUser.setStatus("1");
                insureUser.setPolicyId(insurePolicy.getId());
                y.setIsInsure(1);
                y.insertOrUpdate();
                insureUser.insert();
            }
        }
        return ResultUtil.error("");

    }

    /*测试用生产环境不需要*/
    synchronized private Map requestAgent(Map bodyMap) {
        String data = HttpUtils.sendPost(insuredUrl, setParams(JSONObject.toJSONString(bodyMap), appid, secret), bodyMap);
        Map dataMap = JSONObject.parseObject(data, Map.class);
        InsureLog.builder().requestParam(JSONObject.toJSONString(setParams(JSONObject.toJSONString(bodyMap), appid, secret))).type(1)
                .requestData(JSONObject.toJSONString(bodyMap)).createTime(new Date()).requestType(1).returnBody(data).requestPath(insuredUrl)
                .returnCode(dataMap.get("errcode").toString()).returnMsg(dataMap.get("errmsg").toString()).build().insert();

        if (dataMap.get("errcode").toString().equals("suc")) {
            return dataMap;
        } else {
            if (dataMap.get("errcode").toString().equals("e1") && (dataMap.get("errmsg").toString().equals("出单中") || dataMap.get("errmsg").toString().equals("系统正在处理中，请稍候再试"))) {
                try {
                    Thread.sleep(30000);
                    return requestAgent(bodyMap);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }

            }
            return null;
        }
    }


    /**
     * 生成投保问文件
     *
     * @param ids
     * @return
     */
    public String creaXSSFWorkbook(String[] ids,Integer price) {
        if (ids.length <= 0) {
            return "error";
        }
        LocalDate localDate = LocalDate.now();
        Date date = new Date();
        List<YgglMainEmp> ygglMainEmpList = ygglMainEmpMapper.selectListByIds(ids);
        List list = Lists.newArrayList();
        for (YgglMainEmp y : ygglMainEmpList) {
            String[] arr = new String[]{String.valueOf(y.getId()), y.getName(), "身份证", y.getZjNum(), InsuranceEnum.getEnum(price).getName(), "", "深圳名邦人力资源管理有限公司", "无", InsuranceEnum.getEnum(price).getType()};
            list.add(arr);
        }
        String[] rowName = new String[]{"ID", "name", "ID_type", "ID_number", "Scheme_name", "Date_start", "Branch", "Tricycle_frame_number", "benefit_occupation_category"};

        XSSFWorkbook xssfWorkbook = ExcelUtils.exportExcel(localDate + "_投保" + list.size() + "名人员清单.xlsx", rowName, list);

        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        try {
            xssfWorkbook.write(byteArrayOutputStream);
            byte[] bytes = byteArrayOutputStream.toByteArray();
            String fileBase64 = Base64.byteArrayToBase64(bytes);

            /*body参数 begin*/
            Map bodyMap = Maps.newHashMap();
            bodyMap.put("app", "mall");
            bodyMap.put("name", localDate + "_" + list.size() + "名人员清单.xlsx");
            bodyMap.put("file", fileBase64);
            /*end*/
            String data = HttpUtils.sendPost(uploadUrl, setParams(JSONObject.toJSONString(bodyMap), appid, secret), bodyMap);
            Map<String, Object> dataMap = JSONObject.parseObject(data);
            /*创建日志*/
            InsureLog.builder().requestParam(JSONObject.toJSONString(setParams(JSONObject.toJSONString(bodyMap), appid, secret))).type(4)
                    .requestData(JSONObject.toJSONString(bodyMap)).createTime(new Date()).requestType(1).returnBody(data).requestPath(uploadUrl)
                    .returnCode(dataMap.get("errcode").toString()).returnMsg(dataMap.get("errmsg").toString()).build().insert();
            if (dataMap.get("errcode").toString().equals("suc")) {
                Object o = dataMap.get("data");
                Map<String, Object> map = JSONObject.parseObject(JSONObject.toJSONString(o));
                return map.get("url").toString();
            }
            /*文件流base64*/
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                xssfWorkbook.close();
                byteArrayOutputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        return "suc";

    }

    @PostMapping(value = "/policyList")
    @ApiOperation(value = "3.保单列表", httpMethod = "Post", notes = "保单列表")
    @ApiOperationSupport(order = 2)
    public Result<Object> policyList(@RequestBody PolicyDto policyDto) {
        Map map = Maps.newHashMap();
        List<PolicyDto> list = insureUserMapper.selectPolicyList(policyDto.getPage(), String.valueOf(policyDto.getId()));
        map.put("list", Optional.ofNullable(list).orElse(Lists.newArrayList()));
        map.put("total", insureUserMapper.totalUser(String.valueOf(policyDto.getId())));
        return ResultUtil.data(map);
    }

    @PostMapping(value = "/policyListAdmin")
    @ApiOperation(value = "4.保单列表--运营后台端", httpMethod = "Post", notes = "保单列表--运营后台端")
    @ApiOperationSupport(order = 2)
    public Result<Object> policyListAdmin(@RequestBody cn.timer.api.utils.Page page) {
        Map map = Maps.newHashMap();
        List<PolicyDto> list = insurePolicyMapper.policyList(page);
        map.put("list", list);
        map.put("total", insurePolicyMapper.totalPolicy());
        return ResultUtil.data(map);
    }

    @GetMapping(value = "/policyDetail")
    @ApiOperation(value = "5.保单信息", httpMethod = "Get", notes = "保单信息")
    @ApiOperationSupport(order = 2)
    public Result<Object> policyDetail(@RequestParam("policyId") Integer policyId) {
        PolicyDto policyDto = insurePolicyMapper.getPolicyDetail(policyId);
        if (policyDto == null) {
            return ResultUtil.error("保单不存在");
        }
        return ResultUtil.data(policyDto);
    }

    @PostMapping(value = "/joinUser")
    @ApiOperation(value = "2.增员", httpMethod = "Post", notes = "增员")
    @ApiOperationSupport(order = 2)
    public Result<Object> joinUser(@RequestBody InsureDto insureDto) throws ParseException {

        InsurePolicy insurePolicy = InsurePolicy.builder().id(Integer.parseInt(insureDto.getId())).build().selectById();
        if (insurePolicy == null) {
            return ResultUtil.error("保单不存在");
        }
        Integer price = getPrice(insureDto.getBenefitBasicPlan(),insureDto.getBenefitOccupationCategory());
        LocalDateTime now = LocalDateTime.now();
        String url = creaXSSFWorkbook1(insureDto.getUserIds(), insureDto.getPolicyDateStart(),price);
        log.info("保全增员文件地址", url);
        DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
        Map bodyMap = Maps.newHashMap();
        bodyMap.put("contract_no", insurePolicy.getPolicyNo());
        bodyMap.put("order_status", "1");
        bodyMap.put("callback_plan", "");
        bodyMap.put("file_url", url);
        bodyMap.put("from_source", "API");
        bodyMap.put("import_review_callback", "");
        bodyMap.put("import_callback", "http://vdr75zl.nat.ipyingshe.com/callBack/policy/CallBack");
        bodyMap.put("third_uuid", "IS_" + insurePolicy.getOrgCode() + "_" + dtf2.format(now));
        String responData = HttpUtils.sendPost(batchUrl, setParams(JSONObject.toJSONString(bodyMap), appidq, secretq), bodyMap);
        Map<String, Object> dataMap = JSONObject.parseObject(responData);
        InsureLog.builder().requestParam(JSONObject.toJSONString(setParams(JSONObject.toJSONString(bodyMap), appid, secret))).type(2)
                .requestData(JSONObject.toJSONString(bodyMap)).createTime(new Date()).requestType(1).returnBody(responData).requestPath(batchUrl)
                .returnCode(dataMap.get("errcode").toString()).returnMsg(dataMap.get("errmsg").toString()).build().insert();
        if (dataMap.get("errcode").toString().equals("suc")) {
            Map<String, Object> data = JSONObject.parseObject(JSONObject.toJSONString(dataMap.get("data")));
            Map<String, Object> orderImportInfo = JSONObject.parseObject(JSONObject.toJSONString(data.get("order_import_info")));
            String status = orderImportInfo.get("status").toString();
            if (status.equals("5") || status.equals("6") || status.equals("7")) {
                //TODO 写入日志
                return ResultUtil.error("信息填写错误，请查看日志");
            } else {
                List<YgglMainEmp> ygglMainEmpList = ygglMainEmpMapper.selectListByIds(insureDto.getUserIds());
                QyzxEntInfoM qyzxEntInfoM = QyzxEntInfoM.builder().id(Integer.parseInt(insureDto.getOrgCode())).build().selectById();
                ygglMainEmpList.forEach(y -> {
                    try {
                        InsureUser.builder()
                                .transId("IS_" + insurePolicy.getOrgCode() + "_" + dtf2.format(now))
                                .insuredName(qyzxEntInfoM.getName())
                                .price(price.toString())/*前端获取的保费*/
                                .premium(String.valueOf(price * ygglMainEmpList.size()))
                                .batchNo(data.get("import_uuid").toString())
                                .benefitBasicPlan(insureDto.getBenefitBasicPlan())
                                .policyNo(orderImportInfo.get("contract_no").toString())
                                .userId(y.getId())
                                .orgCode(y.getOrgCode())
                                .benefitOccupationCategory(insureDto.getBenefitOccupationCategory())
                                .insuredMobile(y.getPhone())
                                .benefitOccupationCategory(insureDto.getBenefitOccupationCategory())
                                .insuredNo(y.getZjNum())
                                .insuredEContact(y.getName())
                                .applicantEmployeeList(data.get("file_url").toString())
                                .policyDateStart(dtf3.parse(insureDto.getPolicyDateStart()))
                                .policyDateEnd(dtf3.parse(insureDto.getPolicyDateEnd()))
                                .status("1").insureStatus(3).policyId(insurePolicy.getId()).build().insert();
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                    y.setIsInsure(2);
                    y.insertOrUpdate();
                });
                return ResultUtil.data("核保成功，等待系统更新");
            }
        }
        return ResultUtil.error(dataMap.get("请求异常"));
    }

    /**
     * 生成保全增员文件
     *
     * @param ids
     * @return
     */
    public String creaXSSFWorkbook1(String[] ids, String date,Integer price) {
        if (ids.length <= 0) {
            return "error";
        }
        LocalDate localDate = LocalDate.now();
        List<YgglMainEmp> ygglMainEmpList = ygglMainEmpMapper.selectListByIds(ids);
        List list = Lists.newArrayList();
        for (YgglMainEmp y : ygglMainEmpList) {
            String[] arr = new String[]{String.valueOf(y.getId()), y.getName(), "身份证", y.getZjNum(), InsuranceEnum.getEnum(price).getName(), date, "深圳名邦人力资源管理有限公司", "无", InsuranceEnum.getEnum(price).getType()};
            list.add(arr);
        }
        String[] rowName = new String[]{"ID", "name", "ID_type", "ID_number", "Scheme_name", "Date_start", "Branch", "Tricycle_frame_number", "benefit_occupation_category"};

        XSSFWorkbook xssfWorkbook = ExcelUtils.exportExcel( localDate + "_补充" + list.size() + "名人员清单.xlsx", rowName, list);

        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        try {
            xssfWorkbook.write(byteArrayOutputStream);
            byte[] bytes = byteArrayOutputStream.toByteArray();
            String fileBase64 = Base64.byteArrayToBase64(bytes);

            /*body参数 begin*/
            Map bodyMap = Maps.newHashMap();
            bodyMap.put("media", fileBase64);
            /*end*/
            String data = HttpUtils.sendPost(uploadUrl2, setParams(JSONObject.toJSONString(bodyMap), appidq, secretq), bodyMap);
            Map<String, Object> dataMap = JSONObject.parseObject(data);
            InsureLog.builder().requestParam(JSONObject.toJSONString(setParams(JSONObject.toJSONString(bodyMap), appid, secret))).type(5)
                    .requestData(JSONObject.toJSONString(bodyMap)).createTime(new Date()).requestType(1).returnBody(data).requestPath(uploadUrl2)
                    .returnCode(dataMap.get("errcode").toString()).returnMsg(dataMap.get("errmsg").toString()).build().insert();
            if (dataMap.get("errcode").toString().equals("suc")) {
                Object o = dataMap.get("data");
                Map<String, Object> map = JSONObject.parseObject(JSONObject.toJSONString(o));
                return map.get("file_url").toString();
            }
            /*文件流base64*/
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                xssfWorkbook.close();
                byteArrayOutputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return "suc";
    }

    @GetMapping(value = "/downPolicyFile")
    @ApiOperation(value = "7.下载电子表单", httpMethod = "Get", notes = "下载电子表单")
    @ApiOperationSupport(order = 2)
    public void downPolicyFile(@RequestParam("policyId") String policyId, HttpServletRequest request, HttpServletResponse response) {
        InsurePolicy insurePolicy = InsurePolicy.builder().id(Integer.parseInt(policyId)).build().selectById();
        if(insurePolicy==null|| StringUtils.isNullOrEmpty(insurePolicy.getPolicyFile())){
            return;
        }
        BufferedInputStream in = null;
        OutputStream os = null;
        try {
            response.setContentType("application/octet-stream");
            response.setHeader("content-disposition", "attachment; filename=" + new String((insurePolicy.getPolicyNo()+".pdf").getBytes("UTF8"), "ISO-8859-1"));
            response.setCharacterEncoding("UTF-8");
            URL  url = new URL(insurePolicy.getPolicyFile());
            in = new BufferedInputStream(url.openStream());
            os = response.getOutputStream();
            int i;
            while ((i = in.read()) != -1) {
                os.write(i);
            }
            os.flush();
            in.close();
            response.getOutputStream();
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    @PostMapping(value = "/replaceUser")
    @ApiOperation(value = "6.保全替换", httpMethod = "Post", notes = "保全替换")
    @ApiOperationSupport(order = 2)
    public Result<Object> replaceUser(@RequestBody InsureDto insureDto) {
        InsurePolicy insurePolicy = InsurePolicy.builder().id(Integer.parseInt(insureDto.getId())).build().selectById();
        if (insurePolicy == null) {
            return ResultUtil.error("保单不存在");
        }
        List<InsureUser> oldInsureUserList = insureUserMapper.selectListByIds(insureDto.getOldIds());
        String [] oldUser = new String[oldInsureUserList.size()];
        for (int i=0;i<oldInsureUserList.size();i++) {
            oldUser[i]=oldInsureUserList.get(i).getUserId().toString();
        }
        Integer price = getPrice(insureDto.getBenefitBasicPlan(),insureDto.getBenefitOccupationCategory());
        LocalDateTime now = LocalDateTime.now();
        String url = creaXSSFWorkbook2(insureDto.getUserIds(), insureDto.getPolicyDateStart(),oldUser,insureDto.getBenefitOccupationCategory());
        log.info("保全替换文件地址", url);
        DateTimeFormatter dtf2 = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
        Map bodyMap = Maps.newHashMap();
        bodyMap.put("contract_no", insurePolicy.getPolicyNo());
        bodyMap.put("order_status", "3");
        bodyMap.put("callback_plan", "");
        bodyMap.put("file_url", url);
        bodyMap.put("from_source", "API");
        bodyMap.put("import_review_callback", "");
        bodyMap.put("import_callback", "http://vdr75zl.nat.ipyingshe.com/callBack/policy/CallBack");
        bodyMap.put("third_uuid", "RP_" + insurePolicy.getOrgCode() + "_" + dtf2.format(now));
        String responData = HttpUtils.sendPost(batchUrl, setParams(JSONObject.toJSONString(bodyMap), appidq, secretq), bodyMap);
        Map<String, Object> dataMap = JSONObject.parseObject(responData);
        InsureLog.builder().requestParam(JSONObject.toJSONString(setParams(JSONObject.toJSONString(bodyMap), appid, secret))).type(3)
                .requestData(JSONObject.toJSONString(bodyMap)).createTime(new Date()).requestType(1).returnBody(responData).requestPath(batchUrl)
                .returnCode(dataMap.get("errcode").toString()).returnMsg(dataMap.get("errmsg").toString()).build().insert();
        if (dataMap.get("errcode").toString().equals("suc")) {
            Map<String, Object> data = JSONObject.parseObject(JSONObject.toJSONString(dataMap.get("data")));
            Map<String, Object> orderImportInfo = JSONObject.parseObject(JSONObject.toJSONString(data.get("order_import_info")));
            String status = orderImportInfo.get("status").toString();
            if (status.equals("5") || status.equals("6") || status.equals("7")) {
                //TODO 写入日志
                return ResultUtil.error("信息填写错误，请查看日志");
            } else {
                List<YgglMainEmp> ygglMainEmpList = ygglMainEmpMapper.selectListByIds(insureDto.getUserIds());
                QyzxEntInfoM qyzxEntInfoM = QyzxEntInfoM.builder().id(Integer.parseInt(insureDto.getOrgCode())).build().selectById();
                ygglMainEmpList.forEach(y -> {
                    try {
                        InsureUser.builder()
                                .transId("RP_" + insurePolicy.getOrgCode() + "_" + dtf2.format(now))
                                .insuredName(qyzxEntInfoM.getName())
                                .price(price.toString())/*前端获取的保费*/
                                .premium(String.valueOf(price * ygglMainEmpList.size()))
                                .batchNo(data.get("import_uuid").toString())
                                .benefitBasicPlan(insureDto.getBenefitBasicPlan())
                                .policyNo(orderImportInfo.get("contract_no").toString())
                                .userId(y.getId())
                                .orgCode(y.getOrgCode())
                                .benefitOccupationCategory(insureDto.getBenefitOccupationCategory())
                                .insuredMobile(y.getPhone())
                                .benefitOccupationCategory(insureDto.getBenefitOccupationCategory())
                                .insuredNo(y.getZjNum())
                                .insuredEContact(y.getName())
                                .applicantEmployeeList(data.get("file_url").toString())
                                .policyDateStart(dtf3.parse(insureDto.getPolicyDateStart()))
                                .policyDateEnd(dtf3.parse(insureDto.getPolicyDateEnd()))
                                .status("1").insureStatus(3).policyId(insurePolicy.getId()).build().insert();
                    } catch (ParseException e) {
                        e.printStackTrace();
                    }
                    y.setIsInsure(2);
                    y.insertOrUpdate();
                });
                oldInsureUserList.forEach(o->{
                    o.setReplaceTransId(data.get("import_uuid").toString());
                    o.updateById();
                });

                return ResultUtil.data("核保成功，等待系统更新");
            }
        }
        return ResultUtil.data("成功");
    }
    /**
     * 生成保全替换文件
     *
     * @param ids
     * @return
     */
    public String creaXSSFWorkbook2(String[] ids,String date,String[] oldIds,String type) {
        if (ids.length <= 0||oldIds.length<=0) {
            return "error";
        }
        LocalDate localDate = LocalDate.now();
        List<YgglMainEmp> newList = ygglMainEmpMapper.selectListByIds(ids);
        List<YgglMainEmp> oldList = ygglMainEmpMapper.selectListByIds(oldIds);
        if (newList.size()!=oldList.size()){
            return "error";
        }
        List list = Lists.newArrayList();
        for (int i=0;i<newList.size();i++) {
            String[] arr = new String[]{String.valueOf(oldList.get(i).getId()),
                    oldList.get(i).getName(),
                    "身份证",
                    oldList.get(i).getZjNum(),
                    date.substring(0,10),
                    newList.get(i).getName(),
                    "身份证",
                    newList.get(i).getZjNum(),
                    "","","","","","","无","1",type.equals("63119")?"A类":type.equals("63120")?"B类":"C类"};
            list.add(arr);
        }
        String[] rowName = new String[]{"ID", "name", "ID_type", "ID_number", "Update_date", "replace_name", "replace_ID_type",
                "replace_ID_number", "Date_of_birth","insured_type","insured_relationship","cert_type_related_insured","insured_relationship_id","Branch",
                "Tricycle_frame_number","insured_number","benefit_occupation_category"};

        XSSFWorkbook xssfWorkbook = ExcelUtils.exportExcel( localDate + "_替换" + list.size() + "名人员清单.xlsx", rowName, list);

        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        try {
            xssfWorkbook.write(byteArrayOutputStream);
            byte[] bytes = byteArrayOutputStream.toByteArray();
            String fileBase64 = Base64.byteArrayToBase64(bytes);

            /*body参数 begin*/
            Map bodyMap = Maps.newHashMap();
            bodyMap.put("media", fileBase64);
            /*end*/
            String data = HttpUtils.sendPost(uploadUrl2, setParams(JSONObject.toJSONString(bodyMap), appidq, secretq), bodyMap);
            Map<String, Object> dataMap = JSONObject.parseObject(data);
            InsureLog.builder().requestParam(JSONObject.toJSONString(setParams(JSONObject.toJSONString(bodyMap), appid, secret))).type(6)
                    .requestData(JSONObject.toJSONString(bodyMap)).createTime(new Date()).requestType(1).returnBody(data).requestPath(uploadUrl2)
                    .returnCode(dataMap.get("errcode").toString()).returnMsg(dataMap.get("errmsg").toString()).build().insert();
            if (dataMap.get("errcode").toString().equals("suc")) {
                Object o = dataMap.get("data");
                Map<String, Object> map = JSONObject.parseObject(JSONObject.toJSONString(o));
                return map.get("file_url").toString();
            }
            /*文件流base64*/
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                xssfWorkbook.close();
                byteArrayOutputStream.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return "suc";
    }
    /*获取基础计划的价格*/
    private Integer getPrice(String plan,String type){
       /*
       基础计划
       30万雇主/3万医疗   36968,
       50万雇主/5万医疗	36969,
       80万雇主/10万医疗	36970,
       80万雇主/20万医疗	36971,
       100万雇主/10万医疗 36972
       */
        /*行业类型
        * A类	63119
        * B类	63120
        * C类	63121\
        * */
        switch (plan){
            case "36968":
                if(type.equals("63119")){
                    return 12;
                }else if(type.equals("63120")){
                    return 18;
                }
                break;
            case"36969":
                if(type.equals("63119")){
                    return 15;
                }else if(type.equals("63120")){
                    return 20;
                }else if(type.equals("63121")){
                    return 60;
                }
                break;
            case "36970":
                if(type.equals("63119")){
                    return 18;
                }else if(type.equals("63120")){
                    return 35;
                }else if(type.equals("63121")){
                    return 75;
                }
                break;
            case "36971":
                if(type.equals("63119")){
                    return 25;
                }else if(type.equals("63120")){
                    return 38;
                }
                break;
            case "36972":
                if(type.equals("63119")){
                    return 28;
                }else if(type.equals("63120")){
                    return 40;
                }
                break;
        }
        return 0;
    }


}
